@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2002 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,6 +41,12 @@
 #define isblank(c) ((c) == ' ' || (c) == '\t')
 #endif /* HAVE_ISBLANK */
 
+#ifdef ENABLE_WARNINGS
+#define w(x) x
+#else
+#define w(x)
+#endif /* ENABLE_WARNINGS */
+
 #define d(x)
 
 static void g_mime_parser_class_init (GMimeParserClass *klass);
@@ -96,11 +102,13 @@ struct _GMimeParserPrivate {
 	off_t headers_start;
 	off_t header_start;
 	
-	unsigned int unstep:28;
+	unsigned int unstep:26;
 	unsigned int midline:1;
 	unsigned int seekable:1;
 	unsigned int scan_from:1;
 	unsigned int have_regex:1;
+	unsigned int persist_stream:1;
+	unsigned int respect_content_length:1;
 	
 	GMimeContentType *content_type;
 	struct _header_raw *headers;
@@ -114,6 +122,7 @@ struct _boundary_stack {
 	unsigned int boundarylen;
 	unsigned int boundarylenfinal;
 	unsigned int boundarylenmax;
+	off_t content_end;
 };
 
 static void
@@ -136,10 +145,12 @@ parser_push_boundary (GMimeParser *parser, const char *boundary)
 	} else {
 		s->boundary = g_strdup_printf ("--%s--", boundary);
 		s->boundarylen = strlen (boundary) + 2;
-		s->boundarylenfinal = strlen (s->boundary);
+		s->boundarylenfinal = s->boundarylen + 2;
 	}
 	
 	s->boundarylenmax = MAX (s->boundarylenfinal, max);
+	
+	s->content_end = -1;
 }
 
 static void
@@ -149,7 +160,7 @@ parser_pop_boundary (GMimeParser *parser)
 	struct _boundary_stack *s;
 	
 	if (!priv->bounds) {
-		g_warning ("boundary stack underflow");
+		d(g_warning ("boundary stack underflow"));
 		return;
 	}
 	
@@ -174,7 +185,7 @@ header_raw_find (struct _header_raw *headers, const char *name, off_t *offset)
 	
 	h = headers;
 	while (h) {
-		if (!g_strcasecmp (h->name, name)) {
+		if (!strcasecmp (h->name, name)) {
 			if (offset)
 				*offset = h->offset;
 			return h->value;
@@ -246,6 +257,7 @@ g_mime_parser_init (GMimeParser *parser, GMimeParserClass *klass)
 	
 	parser->priv->scan_from = FALSE;
 	parser->priv->have_regex = FALSE;
+	parser->priv->persist_stream = TRUE;
 }
 
 static void
@@ -403,18 +415,47 @@ g_mime_parser_init_with_stream (GMimeParser *parser, GMimeStream *stream)
 
 
 /**
- * g_mime_parser_set_scan_from:
+ * g_mime_parser_get_persist_stream:
  * @parser: MIME parser object
- * @scan_from: %TRUE to scan From-lines or %FALSE otherwise
  *
- * Sets whether or not @parser should scan mbox-style From-lines.
+ * Gets whether or not the underlying stream is persistant.
+ *
+ * Returns %TRUE if the @parser will leave the content on disk or
+ * %FALSE if it will load the content into memory.
+ **/
+gboolean
+g_mime_parser_get_persist_stream (GMimeParser *parser)
+{
+	g_return_val_if_fail (GMIME_IS_PARSER (parser), FALSE);
+	
+	return (parser->priv->persist_stream && parser->priv->seekable);
+}
+
+
+/**
+ * g_mime_parser_set_persist_stream:
+ * @parser: MIME parser object
+ * @persist: persist attribute
+ *
+ * Sets whether or not the @parser's underlying stream is persistant.
+ *
+ * If @persist is %TRUE, the @parser will attempt to construct
+ * messages/parts whos content will remain on disk rather than being
+ * loaded into memory so as to reduce memory usage. This is the default.
+ *
+ * If @persist is %FALSE, the @parser will always load message content
+ * into memory.
+ *
+ * Note: This attribute only serves as a hint to the @parser. If the
+ * underlying stream does not support seeking, then this attribute
+ * will be ignored.
  **/
 void
-g_mime_parser_set_scan_from (GMimeParser *parser, gboolean scan_from)
+g_mime_parser_set_persist_stream (GMimeParser *parser, gboolean persist)
 {
 	g_return_if_fail (GMIME_IS_PARSER (parser));
 	
-	parser->priv->scan_from = scan_from ? 1 : 0;
+	parser->priv->persist_stream = persist;
 }
 
 
@@ -436,6 +477,62 @@ g_mime_parser_get_scan_from (GMimeParser *parser)
 }
 
 
+/**
+ * g_mime_parser_set_scan_from:
+ * @parser: MIME parser object
+ * @scan_from: %TRUE to scan From-lines or %FALSE otherwise
+ *
+ * Sets whether or not @parser should scan mbox-style From-lines.
+ **/
+void
+g_mime_parser_set_scan_from (GMimeParser *parser, gboolean scan_from)
+{
+	g_return_if_fail (GMIME_IS_PARSER (parser));
+	
+	parser->priv->scan_from = scan_from ? 1 : 0;
+}
+
+
+/**
+ * g_mime_parser_get_respect_content_length:
+ * @parser: MIME parser object
+ *
+ * Gets whether or not @parser is set to use Content-Length for
+ * determining the offset of the end of the message.
+ *
+ * Returns whether or not @parser is set to use Content-Length for
+ * determining the offset of the end of the message.
+ **/
+gboolean
+g_mime_parser_get_respect_content_length (GMimeParser *parser)
+{
+	g_return_val_if_fail (GMIME_IS_PARSER (parser), FALSE);
+	
+	return parser->priv->respect_content_length;
+}
+
+
+/**
+ * g_mime_parser_get_respect_content_length:
+ * @parser: MIME parser object
+ * @use_content_length: %TRUE if the parser should use Content-Length headers or %FALSE otherwise.
+ *
+ * Sets whether or not @parser should respect Content-Length headers
+ * when deciding where to look for the start of the next message. Only
+ * used when the parser is also set to scan for From-lines.
+ *
+ * Most notably useful when parsing broken Solaris mbox files (See
+ * http://www.jwz.org/doc/content-length.html for details).
+ **/
+void
+g_mime_parser_set_respect_content_length (GMimeParser *parser, gboolean respect_content_length)
+{
+	g_return_if_fail (GMIME_IS_PARSER (parser));
+	
+	parser->priv->respect_content_length = respect_content_length ? 1 : 0;
+}
+
+
 /**
  * g_mime_parser_set_header_regex:
  * @parser: MIME parser object
@@ -525,11 +622,8 @@ parser_fill (GMimeParser *parser)
 
 
 static off_t
-parser_offset (GMimeParser *parser, unsigned char *cur)
+parser_offset (struct _GMimeParserPrivate *priv, unsigned char *inptr)
 {
-	struct _GMimeParserPrivate *priv = parser->priv;
-	unsigned char *inptr = cur;
-	
 	if (!inptr)
 		inptr = priv->inptr;
 	
@@ -552,7 +646,7 @@ g_mime_parser_tell (GMimeParser *parser)
 	g_return_val_if_fail (GMIME_IS_PARSER (parser), -1);
 	g_return_val_if_fail (GMIME_IS_STREAM (parser->priv->stream), -1);
 	
-	return parser_offset (parser, NULL);
+	return parser_offset (parser->priv, NULL);
 }
 
 
@@ -613,7 +707,7 @@ parser_step_from (GMimeParser *parser)
 			inptr++;
 			
 			if (len >= 5 && !strncmp (start, "From ", 5)) {
-				priv->from_offset = parser_offset (parser, start);
+				priv->from_offset = parser_offset (priv, start);
 				g_byte_array_append (priv->from_line, start, len);
 				goto got_from;
 			}
@@ -669,7 +763,7 @@ parser_step_from (GMimeParser *parser)
 	header->name = g_strndup (priv->headerbuf, hlen);                 \
 	g_strstrip (header->name);                                        \
 	if (*colon != ':') {                                              \
-		g_warning ("Invalid header: %s", header->name);           \
+		w(g_warning ("Invalid header: %s", header->name));        \
 		header->value = header->name;                             \
 		header->name = g_strdup ("X-Invalid-Header");             \
 	} else {                                                          \
@@ -701,8 +795,8 @@ parser_step_headers (GMimeParser *parser)
 	
 	priv->midline = FALSE;
 	hend = (struct _header_raw *) &priv->headers;
-	priv->headers_start = parser_offset (parser, NULL);
-	priv->header_start = parser_offset (parser, NULL);
+	priv->headers_start = parser_offset (priv, NULL);
+	priv->header_start = parser_offset (priv, NULL);
 	
 	inptr = priv->inptr;
 	inend = priv->inend;
@@ -725,7 +819,7 @@ parser_step_headers (GMimeParser *parser)
 			while (*inptr != '\n')
 				inptr++;
 			
-			if (inptr + 1 >= inend) {
+			if (inptr == inend) {
 				/* we don't have enough data to tell if we
 				   got all of the header or not... */
 				priv->inptr = start;
@@ -748,7 +842,7 @@ parser_step_headers (GMimeParser *parser)
 				} else {
 					priv->midline = FALSE;
 					header_parse (parser, priv, hend);
-					priv->header_start = parser_offset (parser, inptr);
+					priv->header_start = parser_offset (priv, inptr);
 				}
 			} else {
 				priv->midline = TRUE;
@@ -763,7 +857,6 @@ parser_step_headers (GMimeParser *parser)
 	inend = priv->inend;
 	
 	header_backup (priv, inptr, inend - inptr);
-	/*header_parse (priv, hend);*/
 	
  headers_end:
 	
@@ -875,6 +968,8 @@ enum {
 static int
 check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, int len)
 {
+	off_t offset = parser_offset (priv, (unsigned char *) start);
+	
 	if (possible_boundary (priv->scan_from, start, len)) {
 		struct _boundary_stack *s;
 		
@@ -883,10 +978,12 @@ check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, in
 		s = priv->bounds;
 		while (s) {
 			/* we use >= here because From lines are > 5 chars */
-			if (len >= s->boundarylenfinal &&
+			if (offset >= s->content_end &&
+			    len >= s->boundarylenfinal &&
 			    !strncmp (s->boundary, start,
 				      s->boundarylenfinal)) {
-				d(printf ("found end boundary\n"));
+				d(printf ("found %s\n", s->content_end != -1 && offset >= s->content_end ?
+					  "end of content" : "end boundary"));
 				return FOUND_END_BOUNDARY;
 			}
 			
@@ -931,7 +1028,7 @@ parser_scan_content (GMimeParser *parser, GByteArray *content)
 	
 	g_assert (priv->inptr <= priv->inend);
 	
-	inptr = priv->inptr;
+	start = inptr = priv->inptr;
 	
 	do {
 	refill:
@@ -1005,25 +1102,25 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	GMimeStream *stream;
 	off_t start, end;
 	
-	if (priv->seekable)
-		start = parser_offset (parser, NULL);
+	if (priv->persist_stream && priv->seekable)
+		start = parser_offset (priv, NULL);
 	else
 		content = g_byte_array_new ();
 	
 	*found = parser_scan_content (parser, content);
 	if (*found != FOUND_EOS) {
 		/* last '\n' belongs to the boundary */
-		if (priv->seekable)
-			end = parser_offset (parser, NULL) - 1;
+		if (priv->persist_stream && priv->seekable)
+			end = parser_offset (priv, NULL) - 1;
 		else
 			g_byte_array_set_size (content, MAX (content->len - 1, 0));
-	} else if (priv->seekable) {
-		end = parser_offset (parser, NULL);
+	} else if (priv->persist_stream && priv->seekable) {
+		end = parser_offset (priv, NULL);
 	}
 	
 	encoding = g_mime_part_get_encoding (mime_part);
 	
-	if (priv->seekable) {
+	if (priv->persist_stream && priv->seekable) {
 		stream = g_mime_stream_substream (priv->stream, start, end);
 	} else {
 		stream = g_mime_stream_mem_new_with_byte_array (content);
@@ -1176,6 +1273,14 @@ parser_scan_multipart_subparts (GMimeParser *parser, GMimeMultipart *multipart)
 	return found;
 }
 
+static gboolean
+found_immediate_boundary (struct _GMimeParserPrivate *priv)
+{
+	struct _boundary_stack *s = priv->bounds;
+	
+	return !strncmp (s->boundary, priv->inptr, s->boundarylenfinal);
+}
+
 static GMimeObject *
 parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type, int *found)
 {
@@ -1213,15 +1318,16 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 		
 		if (*found == FOUND_BOUNDARY)
 			*found = parser_scan_multipart_subparts (parser, multipart);
-		parser_pop_boundary (parser);
 		
-		/* eat end boundary */
-		parser_skip_line (parser);
-		
-		if (*found == FOUND_END_BOUNDARY)
+		if (*found == FOUND_END_BOUNDARY && found_immediate_boundary (priv)) {
+			/* eat end boundary */
+			parser_skip_line (parser);
 			*found = parser_scan_multipart_postface (parser, multipart);
+		}
+		
+		parser_pop_boundary (parser);
 	} else {
-		g_warning ("multipart without boundary encountered");
+		w(g_warning ("multipart without boundary encountered"));
 		/* this will scan everything into the preface */
 		*found = parser_scan_multipart_preface (parser, multipart);
 	}
@@ -1278,6 +1384,7 @@ parser_construct_message (GMimeParser *parser)
 	struct _GMimeParserPrivate *priv = parser->priv;
 	GMimeContentType *content_type;
 	struct _header_raw *header;
+	int content_length = -1;
 	GMimeMessage *message;
 	GMimeObject *object;
 	int found;
@@ -1289,12 +1396,18 @@ parser_construct_message (GMimeParser *parser)
 	message = g_mime_message_new (FALSE);
 	header = priv->headers;
 	while (header) {
+		if (priv->respect_content_length && !strcasecmp (header->name, "Content-Length"))
+			content_length = strtoul (header->value, NULL, 10);
+		
 		g_mime_object_add_header (GMIME_OBJECT (message), header->name, header->value);
 		header = header->next;
 	}
 	
-	if (priv->scan_from)
+	if (priv->scan_from) {
 		parser_push_boundary (parser, "From ");
+		if (priv->respect_content_length && content_length != -1)
+			priv->bounds->content_end = parser_offset (priv, NULL) + content_length;
+	}
 	
 	content_type = parser_content_type (parser);
 	if (!content_type)
