@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2002 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,17 +28,24 @@
 #include "gmime.h"
 
 
+GQuark gmime_error_quark;
+
+
+static int initialized = FALSE;
+
+
 /**
  * g_mime_init:
  * @flags: initialization flags
  *
  * Initializes GMime.
+ *
+ * Note: Calls #g_mime_charset_map_init() and #g_mime_iconv_init() as
+ * well.
  **/
 void
 g_mime_init (guint32 flags)
 {
-	static int initialized = FALSE;
-	
 	if (initialized)
 		return;
 	
@@ -50,6 +57,8 @@ g_mime_init (guint32 flags)
 	
 	g_mime_iconv_init ();
 	
+	gmime_error_quark = g_quark_from_static_string ("gmime");
+	
 	/* register our default mime object types */
 	g_mime_object_register_type ("*", "*", g_mime_part_get_type ());
 	g_mime_object_register_type ("multipart", "*", g_mime_multipart_get_type ());
@@ -60,3 +69,22 @@ g_mime_init (guint32 flags)
 	g_mime_object_register_type ("message", "news", g_mime_message_part_get_type ());
 	g_mime_object_register_type ("message", "partial", g_mime_message_partial_get_type ());
 }
+
+
+/**
+ * g_mime_shutdown:
+ *
+ * Frees internally allocated tables created in #g_mime_init(). Also
+ * calls #g_mime_charset_map_shutdown() and #g_mime_iconv_shutdown().
+ **/
+void
+g_mime_shutdown (void)
+{
+	if (!initialized)
+		return;
+	
+	g_mime_charset_map_shutdown ();
+	g_mime_iconv_shutdown ();
+	
+	initialized = FALSE;
+}
